@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tclFileName.c 1.32 97/08/19 18:44:03
+ * RCS: @(#) $Id: tclFileName.c,v 1.4 1999/02/03 19:12:26 stanton Exp $
  */
 
 #include "tclInt.h"
@@ -59,7 +59,7 @@ TclPlatformType tclPlatform = TCL_PLATFORM_UNIX;
  */
 
 static char *		DoTildeSubst _ANSI_ARGS_((Tcl_Interp *interp,
-			    char *user, Tcl_DString *resultPtr));
+			    CONST char *user, Tcl_DString *resultPtr));
 static CONST char *	ExtractWinRoot _ANSI_ARGS_((CONST char *path,
 			    Tcl_DString *resultPtr, int offset));
 static void		FileNameCleanup _ANSI_ARGS_((ClientData clientData));
@@ -195,7 +195,7 @@ ExtractWinRoot(path, resultPtr, offset)
 
 Tcl_PathType
 Tcl_GetPathType(path)
-    char *path;
+    CONST char *path;
 {
     Tcl_PathType type = TCL_PATH_ABSOLUTE;
 
@@ -227,7 +227,7 @@ Tcl_GetPathType(path)
 			initialized = 1;
 		    }
 		}
-		if (!TclRegExec(macRootPatternPtr, path, path)
+		if (!TclRegExec(macRootPatternPtr, (char *) path, (char *) path)
 			|| (macRootPatternPtr->startp[2] != NULL)) {
 		    type = TCL_PATH_RELATIVE;
 		}
@@ -249,7 +249,7 @@ Tcl_GetPathType(path)
 			initialized = 1;
 		    }
 		}
-		if (TclRegExec(winRootPatternPtr, path, path)) {
+		if (TclRegExec(winRootPatternPtr, (char *) path, (char *) path)) {
 		    if (winRootPatternPtr->startp[5]
 			    || (winRootPatternPtr->startp[2]
 				    && !(winRootPatternPtr->startp[6]))) {
@@ -295,7 +295,7 @@ Tcl_SplitPath(path, argcPtr, argvPtr)
     CONST char *path;		/* Pointer to string containing a path. */
     int *argcPtr;		/* Pointer to location to fill in with
 				 * the number of elements in the path. */
-    char ***argvPtr;		/* Pointer to place to store pointer to array
+    CONST char ***argvPtr;	/* Pointer to place to store pointer to array
 				 * of pointers to path elements. */
 {
     int i, size;
@@ -341,7 +341,7 @@ Tcl_SplitPath(path, argcPtr, argvPtr)
      * DString plus the argv pointers and the terminating NULL pointer.
      */
 
-    *argvPtr = (char **) ckalloc((unsigned)
+    *argvPtr = (CONST char **) ckalloc((unsigned)
 	    ((((*argcPtr) + 1) * sizeof(char *)) + size));
 
     /*
@@ -685,7 +685,7 @@ SplitMacPath(path, bufPtr)
 char *
 Tcl_JoinPath(argc, argv, resultPtr)
     int argc;
-    char **argv;
+    CONST char * CONST argv[];
     Tcl_DString *resultPtr;	/* Pointer to previously initialized DString. */
 {
     int oldLength, length, i, needsSep;
@@ -912,7 +912,7 @@ char *
 Tcl_TranslateFileName(interp, name, bufferPtr)
     Tcl_Interp *interp;		/* Interpreter in which to store error
 				 * message (if necessary). */
-    char *name;			/* File name, which may begin with "~"
+    CONST char *name;		/* File name, which may begin with "~"
 				 * (to indicate current user's home directory)
 				 * or "~<user>" (to indicate any user's
 				 * home directory). */
@@ -931,7 +931,7 @@ Tcl_TranslateFileName(interp, name, bufferPtr)
 	char **argv;
 	Tcl_DString temp;
 
-	Tcl_SplitPath(name, &argc, &argv);
+	Tcl_SplitPath(name, &argc, (CONST char ***) &argv);
 	
 	/*
 	 * Strip the trailing ':' off of a Mac path
@@ -951,7 +951,7 @@ Tcl_TranslateFileName(interp, name, bufferPtr)
 	    return NULL;
 	}
 	Tcl_DStringInit(bufferPtr);
-	Tcl_JoinPath(argc, argv, bufferPtr);
+	Tcl_JoinPath(argc, (CONST char **) argv, bufferPtr);
 	Tcl_DStringFree(&temp);
 	ckfree((char*)argv);
     } else {
@@ -992,11 +992,11 @@ Tcl_TranslateFileName(interp, name, bufferPtr)
  *----------------------------------------------------------------------
  */
 
-char *
+CONST char *
 TclGetExtension(name)
-    char *name;			/* File name to parse. */
+    CONST char *name;		/* File name to parse. */
 {
-    char *p, *lastSep;
+    CONST char *p, *lastSep;
 
     /*
      * First find the last directory separator.
@@ -1032,15 +1032,12 @@ TclGetExtension(name)
     }
 
     /*
-     * Back up to the first period in a series of contiguous dots.
-     * This is needed so foo..o will be split on the first dot.
+     * In earlier versions, we used to back up to the first period in a series
+     * so that "foo..o" would be split into "foo" and "..o".  This is a
+     * confusing and usually incorrect behavior, so now we split at the last
+     * period in the name.
      */
 
-    if (p != NULL) {
-	while ((p > name) && *(p-1) == '.') {
-	    p--;
-	}
-    }
     return p;
 }
 
@@ -1069,13 +1066,13 @@ static char *
 DoTildeSubst(interp, user, resultPtr)
     Tcl_Interp *interp;		/* Interpreter in which to store error
 				 * message (if necessary). */
-    char *user;			/* Name of user whose home directory should be
+    CONST char *user;		/* Name of user whose home directory should be
 				 * substituted, or "" for current user. */
     Tcl_DString *resultPtr;	/* May be used to hold result.  Must not hold
 				 * anything at the time of the call, and need
 				 * not even be initialized. */
 {
-    char *dir;
+    CONST char *dir;
 
     if (*user == '\0') {
 	dir = TclGetEnv("HOME");
@@ -1559,7 +1556,7 @@ TclDoGlob(interp, separators, headPtr, tail)
 		Tcl_DStringAppend(headPtr, ":", 1);
 	    }
 	    name = Tcl_DStringValue(headPtr);
-	    if (access(name, F_OK) == 0) {
+	    if (TclAccess(name, F_OK) == 0) {
 		if ((name[1] != '\0') && (strchr(name+1, ':') == NULL)) {
 		    Tcl_AppendElement(interp, name+1);
 		} else {
@@ -1590,7 +1587,7 @@ TclDoGlob(interp, separators, headPtr, tail)
 		}
 	    }
 	    name = Tcl_DStringValue(headPtr);
-	    exists = (access(name, F_OK) == 0);
+	    exists = (TclAccess(name, F_OK) == 0);
 	    for (p = name; *p != '\0'; p++) {
 		if (*p == '\\') {
 		    *p = '/';
@@ -1610,7 +1607,7 @@ TclDoGlob(interp, separators, headPtr, tail)
 		}
 	    }
 	    name = Tcl_DStringValue(headPtr);
-	    if (access(name, F_OK) == 0) {
+	    if (TclAccess(name, F_OK) == 0) {
 		Tcl_AppendElement(interp, name);
 	    }
 	    break;
