@@ -3359,7 +3359,7 @@ Tcl_UtfToTitle(str)
     int len = strlen(str);
     int i;
     if (len>0) {
-	str[0] = toupper(UCHAR(str[i]));
+	str[0] = toupper(UCHAR(str[0]));
     }
     for (i = 1; i < len; i++) {
 	str[i] = tolower(UCHAR(str[i]));
