@@ -12,7 +12,7 @@
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 #
-# SCCS: @(#) safe.tcl 1.2 98/06/27 18:11:25
+# RCS: @(#) $Id: safe.tcl,v 1.4 1998/11/11 02:39:31 welch Exp $
 
 #
 # The implementation is based on namespaces. These naming conventions
@@ -32,12 +32,6 @@ namespace eval ::safe {
 	    interpAddToAccessPath interpFindInAccessPath \
 	    setLogCmd ;
 
-# Proto/dummy declarations for auto_mkIndex
-proc ::safe::interpCreate {} {}
-proc ::safe::interpInit {} {}
-proc ::safe::interpConfigure {} {}
-
-
     ####
     #
     # Setup the arguments parsing
@@ -480,7 +474,7 @@ proc ::safe::interpAddToAccessPath {slave path} {
 		{source [file join $tcl_library init.tcl]}}\
 		msg]} {
 	    if {![catch {::interp eval $slave\
-		    {source -rsrc init}} $msg]} {
+		    {source -rsrc init}} msg]} {
 		return $slave
 	    }
 	    Log $slave "can't source init.tcl ($msg)";
@@ -742,9 +736,16 @@ proc ::safe::setLogCmd {args} {
     proc AliasSource {slave args} {
 
 	set argc [llength $args];
-	# Allow only "source filename"
-	# (and not mac specific -rsrc for instance - see comment in ::init
-	# for current rationale)
+	# Allow only "source filename" and "source -rsrc filename"
+	if {($argc == 2) && ([lindex $args 0] == "-rsrc")} {
+	    if {[catch {::interp invokehidden $slave source -rsrc \
+		    [lindex $args 1]} msg]} {
+		Log $slave $msg;
+		return -code error "script error: $msg";
+	    }
+	    return $msg
+	}
+
 	if {$argc != 1} {
 	    set msg "wrong # args: should be \"source fileName\""
 	    Log $slave "$msg ($args)";
