@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tclWinPort.h 1.53 97/07/30 14:12:17
+ * RCS: @(#) $Id: tclWinPort.h,v 1.6 1998/09/30 23:57:18 stanton Exp $
  */
 
 #ifndef _TCLWINPORT
@@ -36,6 +36,11 @@
 #include <windows.h>
 #undef WIN32_LEAN_AND_MEAN
 
+#ifdef BUILD_tcl
+# undef TCL_STORAGE_CLASS
+# define TCL_STORAGE_CLASS DLLEXPORT
+#endif
+
 /*
  * Define EINPROGRESS in terms of WSAEINPROGRESS.
  */
@@ -57,11 +62,12 @@
  * use by tclAlloc.c.
  */
 
-#define TclpSysAlloc(size, isBin)	((void*)GlobalAlloc(GMEM_FIXED, \
-					    (DWORD)size))
-#define TclpSysFree(ptr)		(GlobalFree((HGLOBAL)ptr))
-#define TclpSysRealloc(ptr, size)	((void*)GlobalReAlloc((HGLOBAL)ptr, \
-					    (DWORD)size, 0))
+#define TclpSysAlloc(size, isBin)	((void*)HeapAlloc(GetProcessHeap(), \
+					    (DWORD)0, (DWORD)size))
+#define TclpSysFree(ptr)		(HeapFree(GetProcessHeap(), \
+					    (DWORD)0, (HGLOBAL)ptr))
+#define TclpSysRealloc(ptr, size)	((void*)HeapReAlloc(GetProcessHeap(), \
+					    (DWORD)0, (LPVOID)ptr, (DWORD)size))
 
 /*
  * The default platform eol translation on Windows is TCL_TRANSLATE_CRLF:
@@ -211,7 +217,7 @@
  * EDEADLK as the same value, which confuses Tcl_ErrnoId().
  */
 
-#ifdef _MSC_VER
+#if defined (_MSC_VER) || defined (__GNUC__)
 #    define environ _environ
 #    define hypot _hypot
 #    define exception _exception
@@ -381,6 +387,8 @@ EXTERN int		TclpAccess _ANSI_ARGS_((CONST char *path,
  * Declarations for Windows specific functions.
  */
 
+EXTERN void		TclWinAddProcess _ANSI_ARGS_((HANDLE hProcess,
+			    DWORD id));
 EXTERN void		TclWinConvertError _ANSI_ARGS_((DWORD errCode));
 EXTERN void		TclWinConvertWSAError _ANSI_ARGS_((DWORD errCode));
 EXTERN struct servent * PASCAL FAR
@@ -393,4 +401,53 @@ EXTERN u_short PASCAL FAR
 			TclWinNToHS _ANSI_ARGS_((u_short ns));
 EXTERN int PASCAL FAR	TclWinSetSockOpt _ANSI_ARGS_((SOCKET s, int level,
 		            int optname, const char FAR * optval, int optlen));
+
+/*
+ * Used by the rType field below
+ */
+
+#define FILE_TYPE_COM		0x1006
+#define FILE_TYPE_CONSOLE	0x1007
+
+/*
+ * This structure describes per-instance data for a file reader thread
+ */
+
+typedef struct TclWinReaderInfo {
+    HANDLE postWin;		/* Window to post to when data is available */
+    DWORD postMsg;		/* Message to post when data is available */
+    int readyMask;		/* Events that need to be checked on */
+    int eof;			/* Are we at the end of file? */ 
+    int async;			/* Asynchronous or synchronous? */
+    int overlappedIO;		/* Use overlapped or non-overlapped IO */
+    OVERLAPPED rOverResult;	/* Overlapped read result */
+    HANDLE rSemaphore;		/* Semaphore for read data availability */
+    HANDLE rEvent;		/* Event for read data availability */
+    HANDLE rGoEvent;		/* Event used to allow console input to
+				 * go forward.  By holding off, an extension
+				 * like expect can set the console mode.
+				 * This is useful for disabling echo */
+    HANDLE rHandle;		/* Handle for reading pipe */
+    DWORD rType;		/* Win32 type for the handle */ 
+    int rHandleClosed;		/* Set if the handle was closed */
+    HANDLE rThread;		/* Reader thread handle if running */
+    char *rData;		/* Buffer with currently available data */
+    int rDataLen;		/* Length of currently available data */
+    int rDataPos;		/* Main thread reader position */
+    int refCount;
+} TclWinReaderInfo;
+
+EXTERN TclWinReaderInfo * TclWinReaderNew(HANDLE postWin, DWORD postMsg,
+			    HANDLE file);
+EXTERN void		TclWinReaderFree(TclWinReaderInfo *infoPtr);
+EXTERN int		TclWinReaderInput(TclWinReaderInfo *infoPtr, char *buf,
+			    int bufSize, int *errorCode);
+EXTERN int		TclWinReaderStart(TclWinReaderInfo *infoPtr,
+			    LPTHREAD_START_ROUTINE threadProc);
+EXTERN DWORD CALLBACK	TclWinReaderThread (LPVOID arg);
+
+
+# undef TCL_STORAGE_CLASS
+# define TCL_STORAGE_CLASS DLLIMPORT
+
 #endif /* _TCLWINPORT */
